<?php
namespace Zeedhi\Translate\Controller;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\Manager;

class Project extends \Teknisa\Libs\Controller\CRUD {
	const PROJECT_COLUMN = 'PROJECT';
	const STATUS_COLUMN = 'STATUS';
	const TYPE_COLUMN = 'EXTERNALTYPE';
	const CONTROL_COLUMN = 'CONTROL';
	const STATUS = 'A';
	const TYPE = 'P';
	const CONTROL = array('C', 'X');

	protected $dataSourceName = 'project';
	protected $dataSourceManager;
	protected $resourceProject;
	protected $crudService;

	public function __construct($crudService, $environment, $resourceProject, $dataSourceManager){
		$this->crudService = $crudService;
		$this->resourceProject = $resourceProject;
		$this->dataSourceManager = $dataSourceManager;
		parent::__construct($crudService, $environment);
	}

	public function find(DTO\Request\Filter $request, DTO\Response $response) {
		$filter = $request->getFilterCriteria();
		$filter = $this->findFilterCriteria($filter);
		$filter->setDataSourceName($this->dataSourceName);
		$dataSet = $this->dataSourceManager->findBy($filter);
		$response->addDataSet($dataSet);
	}

	private function findFilterCriteria($filter){
		$conditions = $filter->getConditions();
		if(!empty($conditions)){
			foreach ($conditions as &$condition) {
				if ($condition['columnName'] == 'MANAGER_ID') {
					$page = $filter->getPage();
					$pageSize = $filter->getPageSize();
					$rows = $this->resourceProject->find($condition["value"]);
					$projectValue = array();
					foreach($rows as &$row){
						array_push($projectValue, $row[self::PROJECT_COLUMN]);
					}
					$condition['columnName'] = 'ID';
					$condition['value'] = $projectValue;
					$filter = new FilterCriteria($this->dataSourceName, $conditions, $page, $pageSize);
				}
			}
		}
		$filter->addCondition(self::TYPE_COLUMN, self::TYPE);
		$filter->addCondition(self::STATUS_COLUMN, self::STATUS);
		$filter->addCondition(self::CONTROL_COLUMN, FilterCriteria::NOT_IN, self::CONTROL);
		return $filter;
	}

	public function findLanguagesByProjectId(DTO\Request\Filter $request, DTO\Response $response){
		$this->dataSourceName = "languagesByProjectId";
		$newFilterCriteria = $this->crudService->populateParameterBag($request->getFilterCriteria(), ['PROJECT_ID']);
		$newRequest = new DTO\Request\Filter(
			$newFilterCriteria, 
			$request->getMethod(), 
			$request->getRoutePath(), 
			$request->getUserId()
		);
		$newFilterCriteria->setDataSourceName($this->dataSourceName);
		$newRequest->setParameter('dbname', $this->dataSourceName);
		$rows = $this->dataSourceManager->findBy($newFilterCriteria)->getRows();
		$languages = array_column($rows, 'LANGUAGES' );
		$languagesStr = implode(',', $languages);
		$languagesUnique = array_unique(explode(',', $languagesStr));
		
		$this->dataSourceName = 'language';
		$newFilterCriteria = new FilterCriteria(
			$this->dataSourceName, 
			[[
				'columnName' => 'ID',
				'operator' => 'IN',
				'value' => $languagesUnique
			]]
		);
		$rows = $this->dataSourceManager->findBy($newFilterCriteria)->getRows();
		
		$response->addDataSet(new DataSet('languagesByProjectId', $rows));
		
	}

}