<?php

namespace Zeedhi\Translate\Controller;

require_once __DIR__.'/../Util/translate/google/GoogleTranslate.php';

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\Session\Session;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\DevOps\Model\Translator\Entities\ZhWords;
use Zeedhi\DevOps\Model\Translator\Entities\ZhWordsTrl;
use Zeedhi\DevOps\Model\Translator\Entities\ZhLanguage;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class WordsTrl extends Crud {
    protected $dataSourceName = 'wordsTranslated';

    /** @var EntityManager $entityManager */
    protected $entityManager;

    /** @var Session $session */
    protected $session;

    /** @var ParameterBag $parameterBag */
    protected $parameterBag;

    protected $LanguageTo;
    protected $LanguageFrom;
    private $translatedText = array();

    public function __construct(Manager $manager, Session $session, EntityManager $entityManager, ParameterBag $parameterBag) {
        $this->gt = new \GoogleTranslate();
        /*@todo Use project config file*/
        $this->gt->setRequestType('curl');
        $this->gt->setApiVersion("2");
        $this->gt->setUrl("https://www.googleapis.com/language/translate/v2?");
        $this->gt->setApiKey('AIzaSyCgI0RUf-xWiztETGcHwSCkFjT5QI2fmnU');
        $this->gt->setFormat('html');

        $this->session = $session;
        $this->entityManager = $entityManager;
        $this->parameterBag = $parameterBag;
        parent::__construct($manager);
    }

    private function getWordsIdByProjectSql($projectIds){
        $projectIds = implode(',', $projectIds);
        $sql = <<<SQL
            SELECT
                wrdtrl.*,
                wrd.birt,
                wrd.value words_value,
                lang.language_iso,
                lang.name language_name,
                o.nmoperador
            FROM
                zh_words wrd
                JOIN zh_words_trl wrdtrl ON wrdtrl.words_id = wrd.id
                JOIN zh_language lang ON wrdtrl.language_id = lang.id
           LEFT JOIN operador O ON O.cdoperador = wrdtrl.approved_by
            where
                wrd.id in (
                    SELECT
                        words_id
                    FROM
                        zh_words_repository wrdrep
                    WHERE
                        wrdrep.repository_id IN (
                            SELECT
                                repository_id
                            FROM
                                zh_project_repository rep
                            WHERE
                                rep.project_id IN ($projectIds)
                        )
            )
SQL;
        return $sql;
    }

    private function findByProject($projectIds, $filterCriteria){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();

        $conditions = Utilities::createQueryFiltered($filterCriteria->getConditions());
        $sql = $this->getWordsIdByProjectSql($projectIds);

        $query = $connection->createQueryBuilder()
            ->select('*')
            ->from('(' . $sql . ')')
            ->where('1 = 1 '. $conditions)
            ->setFirstResult($filterCriteria->getFirstResult())
            ->setMaxResults($filterCriteria->getPageSize());

        $rows = $query->execute()->fetchAll();
        foreach ($rows as &$row) {
            $row['__is_new'] = false;
        }
        return $rows;
    }

    public function find(Request\Filter $request, Response $response) {
        $filterData = $request->getFilterCriteria();
        $projects = array();
        $newConditions = array();
        foreach ($filterData->getConditions() as $condition) {
            if ($condition['columnName'] === 'PROJECT_ID') {
                $projects = $condition['value'];
            } else {
                $newConditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $this->dataSourceName,
            $newConditions,
            $filterData->getPage(),
            $filterData->getPageSize()
        );

        if(count($projects)){
            $dataSet = new DataSet($this->dataSourceName, $this->findByProject($projects, $newFilterCriteria));
        } else {
            $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        }
        $response->addDataSet($dataSet);
    }

    public function translateWordsByGoogle(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $repositoryId = $row['repositoryId'];
        $organizationId = $row['organizationId'];
        $languageFromId = $row['languageFrom'];
        $languageToId = $row['languageTo'];
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $this->setSession($row['userId'], $organizationId);

            /** @var \Zeedhi\DevOps\Model\Translator\Entities\ZhLanguage $languageFrom */
            $languageFrom = $this->entityManager->find(ZhLanguage::class, $languageFromId);

            /** @var \Zeedhi\DevOps\Model\Translator\Entities\ZhLanguage $languageTo */
            $languageTo = $this->entityManager->find(ZhLanguage::class, $languageToId);

            $this->setLanguagesToTranslate($languageFrom->getLanguageIso(), $languageTo->getLanguageIso());

            $sql = $this->getSqlToGetRepositoryWords($organizationId, $repositoryId, $languageToId);
            $object_words = $connection->fetchAll($sql);
            if(count($object_words) > 0){
                $this->translateText($object_words);
                $this->saveTranslatedText($object_words, $languageTo, $languageToId);
                $connection->commit();
            } else {
                throw new \Exception("Todas ja estao traduzidas (org: ".$organizationId.", rep: ".$repositoryId.", lang: ".$languageToId.").");
            }
        } catch(\Exception $e) {
            $connection->rollback();
            echo $e->getMessage();
        }
    }

    protected function setSession($userId, $organizationId){
        $this->session->set('USER_ID', $userId);
        $this->session->set('ORGANIZATION_ID', $organizationId);
    }

    protected function setLanguagesToTranslate($languageFromIso, $languageToIso){
        $this->LanguageFrom = $languageFromIso;
        $this->LanguageTo = $languageToIso;
    }

    protected function getSqlToGetRepositoryWords($organizationId, $repositoryId, $language){
        $sql = <<<SQL
SELECT * FROM ZH_WORDS WHERE ORGANIZATION_ID = $organizationId
AND ZH_WORDS.ID IN (SELECT WORDS_ID FROM ZH_WORDS_REPOSITORY WHERE REPOSITORY_ID = $repositoryId)
AND NOT EXISTS (SELECT WORDS_ID FROM ZH_WORDS_TRL WHERE LANGUAGE_ID = $language AND ZH_WORDS_TRL.WORDS_ID = ZH_WORDS.ID)
AND ZH_WORDS.ID NOT IN (SELECT ID
                          FROM ZH_WORDS
                         WHERE IS_NUMBER(VALUE) = 'TRUE')
SQL;
        return $sql;
    }

    protected function translateText($object_words) {
        $i = 0;
        $str_array = 0;
        $untranslatedText = array();
        foreach ($object_words as $words) {
            $str_array += strlen($words['VALUE']);
            if ($str_array > 1000) {
                $this->setTranslatedTextByGoogle($untranslatedText);
                $str_array = strlen($words['VALUE']);
                $untranslatedText = array();
            }
            $untranslatedText[$i] = $words['VALUE'];
            $i++;
        }
        $this->setTranslatedTextByGoogle($untranslatedText);
    }

    protected function scapeParameters($wordsArray) {
        $scapedWordsArray = preg_replace('/\\t/', '<tab>', $wordsArray);
        $scapedWordsArray = preg_replace('/\\n/', '<br>', $scapedWordsArray);
        $scapedWordsArray = preg_replace('/__[^__]*__/', '<span class="notranslate">$0</span>', $scapedWordsArray);
        $scapedWordsArray = preg_replace('/{{[^}}]*}}/', '<span class="notranslate">$0</span>', $scapedWordsArray);
        return $scapedWordsArray;
    }

    protected function unscapeParameters($scapedWordsArray) {
        $wordsArray = preg_replace('/<tab>/', '\t', $scapedWordsArray);
        $wordsArray = preg_replace('/<br>/', '\n', $wordsArray);
        $wordsArray = preg_replace('/<span class="notranslate">__/', '__', $wordsArray);
        $wordsArray = preg_replace('/__<\/span>/', '__', $wordsArray);
        $wordsArray = preg_replace('/<span class="notranslate">{{/', '{{', $wordsArray);
        $wordsArray = preg_replace('/}}<\/span>/', '}}', $wordsArray);
        return html_entity_decode($wordsArray);
    }

    /**
     * Call GTranslate class
     * Url example /*https://www.googleapis.com/language/translate/v2?key=AIzaSyCergqmYJhkW0WmdJrib7RA6cThft3eLcE&source=en&target=de&format=text&q=Row successfull save.&q={%251} Columns successfully imported.&q=No columns.&q={%251} Contraints successfully imported&q=No constraints.&q=Pk not found.&q=Couldn't find %251 field.&q=No update on %251.&q=Success save.&q=Running Project id = %251
     *
     * return string
     *
     */
    protected function setTranslatedTextByGoogle($array_text) {
        $lang = $this->LanguageFrom.'_to_'.$this->LanguageTo;
        $scapedText = $this->scapeParameters($array_text);
        $translatedArray = $this->gt->{$lang}($scapedText);
        $iarray = count($this->translatedText);
        foreach ($translatedArray as $words) {
            $translatedText = $this->unscapeParameters($words->translatedText);
            $this->translatedText[$iarray] = $translatedText;
            $iarray++;
        }
    }

    protected function getSqlToValidateWordsTrl($wordsId, $languageId) {
        $sql = <<<SQL
        SELECT ID
          FROM ZH_WORDS_TRL
         WHERE WORDS_ID = $wordsId
           AND LANGUAGE_ID = $languageId
SQL;
        return $sql;
    }

    /**
     * Insert translated words in WordsTrl.
     *
     * @return void
     */
    protected function saveTranslatedText($object_words, $language, $languageId) {
        $i = 0;
        foreach ($object_words as $words) {
            $newlabel = $this->translatedText[$i];
            if (!empty($newlabel)) {
                $sql = $this->getSqlToValidateWordsTrl($words['ID'], $languageId);
                $records = $this->entityManager->getConnection()->fetchAll($sql);
                if(empty($records)) {
                    try {
                        $newlabel = ucfirst($newlabel);
                    } catch(\Exception $e) {}
                    $words_Entity = $this->entityManager->getRepository(ZhWords::class)->findOneBy(array('value' => $words['VALUE']));
                    $wordtrl = new ZhWordsTrl();
                    $wordtrl->setLanguage($language);
                    $wordtrl->setWords($words_Entity);
                    $wordtrl->setValue($newlabel);
                    $wordtrl->setStatus('T');
                    $wordtrl->setPrimaryValue($newlabel);
                    $this->entityManager->persist($wordtrl);
                }
            }
            $i++;
        }
        $this->entityManager->flush();
    }

    public function getWordsByRepository(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $connection = $this->entityManager->getConnection();
        $organizationId = $row['organizationId'];
        $repositoryId = $row['repositoryId'];
        try {
            $repository = $this->repositoryExists($organizationId, $repositoryId);
            $repository = [
                'wordsPath' => $repository->getWordsPath(),
                'messagesFile' => $repository->getMessagesFile(),
                'messagesStructure' => explode(";", $repository->getMessagesStructure())
            ];
            $sql = $this->getWordsByRepositorySql($organizationId, $repositoryId);
            $words = $connection->fetchAll($sql);
            $birtSql = $this->getWordsByRepositorySql($organizationId, $repositoryId, true);
            $birtWords = $connection->fetchAll($birtSql);
            echo(json_encode([ 'repository' => $repository, 'json' => $words, 'birt' => $birtWords ]));
            die();
        } catch(\Exception $e) {
            echo $e->getMessage();
        }
    }

    protected function repositoryExists($organizationId, $repositoryId){
         $filter = array(
            'id' => $repositoryId,
            'organizationId' => $organizationId
        );
        $repository = $this->entityManager->getRepository('Zeedhi\DevOps\Model\Translator\Entities\ZhRepository')->findOneBy($filter);
        if($repository == null){
            throw new \Exception("Nonexistent repository");
        }
        return $repository;
    }

    protected function getWordsByRepositorySql($organizationId, $repositoryId, $birt = false){
        $birtClause = $birt ? "Wrd.BIRT = 'Y' AND" : '';
        $sql = 'SELECT Wrd.VALUE WORDS_VALUE, Lang.NAME LANGUAGE_NAME, Lang.SHORT_NAME LANGUAGE_ISO, WrdTrl.VALUE';
        $sql .= ' FROM zh_words Wrd JOIN zh_words_trl WrdTrl ON WrdTrl.words_id = Wrd.id';
        $sql .= ' JOIN zh_language Lang ON WrdTrl.language_id = Lang.id';
        $sql .= " WHERE $birtClause EXISTS ( SELECT * FROM zh_words_repository WrdRep ";
        $sql .= ' WHERE WrdTrl.words_id = WrdRep.words_id AND EXISTS ( SELECT * FROM zh_repository Rep';
        $sql .= " WHERE WrdRep.repository_id = Rep.id AND Rep.id = $repositoryId AND (Rep.organization_id = $organizationId OR Rep.is_public = 'Y')))";
        return $sql;
    }

    public function getWordsByProject(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $connection = $this->entityManager->getConnection();
        $organizationId = $row['organizationId'];
        $projectId = $row['projectId'];
        try {
            $this->isProjectExist($organizationId, $projectId);
            $sql = $this->getWordsByProjectSql($organizationId, $projectId);
            $words = $connection->fetchAll($sql);
            $birtSql = $this->getWordsByProjectSql($organizationId, $projectId, true);
            $birtWords = $connection->fetchAll($birtSql);
            echo(json_encode([ 'json' => $words, 'birt' => $birtWords ]));
            die();
        } catch(\Exception $e) {
            echo $e->getMessage();
        }
    }

    public function getWordsByModules(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $connection = $this->entityManager->getConnection();
        $organizationId = $row['organizationId'];
        $data = array();
        try {
            foreach($row['products'] as $product) {
                $productId = $product['id'];
                $projectId = $this->getProjectIdByProductId($organizationId, $productId);
                $sql = $this->getWordsByProjectSql($organizationId, $projectId);
                $words = $connection->fetchAll($sql);
                $birtSql = $this->getWordsByProjectSql($organizationId, $projectId, true);
                $birtWords = $connection->fetchAll($birtSql);
                if(!empty($words) || !empty($birtWords)) {
                    $data[$productId] = array(
                        'json' => $words, 
                        'birt' => $birtWords
                    );
                    if(!empty($product['repositoryId'])) {
                        $repositorySql = $this->getRepositorySql($product['repositoryId']);
                        $repository = $connection->fetchAll($repositorySql);
                        $data[$productId]['repository'] = empty($repository) ? null : $repository[0];
                    }
                }
            }
            $response->addDataSet(new DataSet('getWordsByModules', $data));
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function getProjectIdByProductId($organizationId, $productId) {
         $filter = array(
            'id' => $productId,
            'organization' => $organizationId
        );
        $product = $this->entityManager->getRepository('Zeedhi\DevOps\Model\Configurationmanager\Entities\Product')->findOneBy($filter);
        if(empty($product) || empty($product->getProject())) {
            throw new \Exception("Nonexistent or inactive product: $productId");
        }
        return $product->getProject()->getId();
    }

    protected function isProjectExist($organizationId, $projectId){
         $filter = array(
            'id' => $projectId,
            'organization' => $organizationId
        );
        $project = $this->entityManager->getRepository('Zeedhi\DevOps\Model\Projectmanagement\Entities\Project')->findOneBy($filter);
        if($project == null || $project->getStatus() != 'A'){
            throw new \Exception("Nonexistent or inactive project");
        }
        return $project;
    }

    protected function getWordsByProjectSql($organizationId, $projectId, $birt = false){
        $birtClause = $birt ? "Wrd.BIRT = 'Y' AND" : "";
        $sql = "SELECT Wrd.VALUE WORDS_VALUE, Lang.NAME LANGUAGE_NAME, Lang.SHORT_NAME LANGUAGE_ISO, WrdTrl.VALUE";
        $sql .= " FROM zh_words Wrd";
        $sql .= " JOIN zh_words_trl WrdTrl ON WrdTrl.words_id = Wrd.id";
        $sql .= " JOIN zh_language Lang ON WrdTrl.language_id = Lang.id";
        $sql .= " WHERE $birtClause EXISTS";
        $sql .= " (SELECT * FROM zh_words_repository WrdRep WHERE WrdTrl.words_id = WrdRep.words_id AND EXISTS";
        $sql .= " (SELECT * FROM zh_project_repository PrjRep";
        $sql .= " JOIN zh_repository Rep ON Rep.id = PrjRep.repository_id";
        $sql .= " WHERE WrdRep.repository_id = PrjRep.repository_id AND PrjRep.project_id = $projectId";
        $sql .= " AND Wrd.TABLE_TRL_ID IS NULL AND (Rep.organization_id = $organizationId OR Rep.is_public = 'Y')))";
        return $sql;
    }

    protected function getRepositorySql($repositoryId) {
        return "SELECT RP.ID, RP.URL, RP.CREATED, RP.UPDATED, RP.CREATED_BY, RP.UPDATED_BY, RP.FRONT_PATH, RP.IS_PUBLIC, 
                       RP.DEFAULT_LANGUAGE, RP.NEW_STRUCTURE, RP.WORDS_PATH, RP.MESSAGES_FILE, RP.MESSAGES_STRUCTURE, 
                       LANG.NAME AS DEFAULT_LANGUAGE_NAME, RL.LANGUAGES, RP.ORGANIZATION_ID AS NRORG, RP.IS_ACTIVE 
                  FROM ZH_REPOSITORY RP 
             LEFT JOIN (SELECT RP.ID AS REPOSITORY_ID, 
                               RTRIM(XMLAGG(XMLELEMENT(E, RL.LANGUAGE_ID, ',').EXTRACT('//text()')).GETCLOBVAL(), ',') AS LANGUAGES 
                          FROM ZH_REPOSITORY RP 
                          JOIN ZH_REPOSITORY_LANGUAGE RL ON RP.ID = RL.REPOSITORY_ID 
                      GROUP BY RP.ID ) RL ON RP.ID = RL.REPOSITORY_ID 
                  JOIN ZH_LANGUAGE LANG ON RP.DEFAULT_LANGUAGE = LANG.ID 
                 WHERE RP.ID = $repositoryId";
    }

    public function save(Request\DataSet $request, Response $response) {
        try {
            parent::save($request, $response);
            $rows = $request->getDataSet()->getRows();
            $ids  = [];
            foreach($rows as $row) {
                $ids[] = $row['ID'];
            }
            
            $sql = "UPDATE ZH_WORDS_TRL 
                       SET STATUS = 'A',
                           APPROVED_BY = :APPROVED_BY,
                           APPROVED_AT = SYSDATE
                     WHERE ID IN (" . join(',', $ids) . ")";
            $params = array (
                'APPROVED_BY' => InstanceManager::getInstance()->getService('environment')->getCdOperador()
            );
            
            $this->entityManager->getConnection()->executeQuery($sql, $params);
            $this->entityManager->flush();
            $response->addNotification(new Response\Notification('Successfully approved!'));
        } catch(\Exception $e) {
            throw $e;
        }
    }
}

