module.exports = function(grunt) {

    var srcFiles = ['assets/js/config/*.js', 'assets/js/services/TranslateMessageService.js','assets/js/services/*.js', 'assets/js/controllers/*.js'];
    var helpersFiles = ['assets/tests/unit/helpers/**/*.js'];
    var specsFiles = ['assets/tests/unit/specs/**/*.spec.js'];
    var libFiles = [
        'zh-preferences/dist/preferences.js',
        'zh-filter-preferences/dist/filter-preferences.js',
        'zhuaclfrontend/assets/dist/main.js',
        'zhulog/dist/main.js',
        'tek-libraries/dist/main.js',
        'zh-cache-bypass/src/CacheBypass.js'
    ];

    function getLibPaths() {
        return libFiles.map(function(fileName) {
            return 'bower_components/' + fileName;
        });
    }

    grunt.initConfig({
        concat: {
            zh: {
                options: {
                    process: function(src, filepath) {
                        return '\n' + '// FILE: ' + filepath + '\n' + src;
                    }
                },
                src: srcFiles,
                dest: 'assets/dist/main.js'
            },
            buildLibs: {
                options: {
                    process: function (src, filepath) {
                        return '// FILE: ' + filepath + '\n' + src + '\n';
                    }
                },
                src: getLibPaths(),
                dest: 'assets/dist/libs.js'
            }
        },
        jshint: {
            all: ['GruntFile.js', srcFiles, specsFiles],
            options: {
				evil: true,
				esversion: 6
			}
        },
        jasmine: {
            pivotal: {
                src: srcFiles,
                options: {
                    helpers: helpersFiles,
                    specs: specsFiles,
                    template: require('grunt-template-jasmine-istanbul'),
                    templateOptions: {
                        coverage: 'coverage/coverage.json',
                        report: 'coverage'
                    }
                }
            }
        },
        uglify: {
            options: {
                mangle: false,
                compress: true,
                report: 'min',
                banner: '/*! <%= grunt.template.today("dd-mm-yyyy") %> */\n'
            },
            dist: {
                files: {
                    'assets/dist/main.min.js': srcFiles.concat()
                }
            }
        },
        zhIdGenerator: {
            options: {
                jsonPath: ["../mobile/assets/json"]
            }
        },
        buildAclStruct : {
            options: {
                menuConfigFile: "./menuConfig.json"
            }
        },
        buildEnvironment: {
            development: {
                configFile: '../config/environmentDevelopment.json'
            },
            deploy: {
                configFile: '../config/environmentDeploy.json'
            },
            test: {
                configFile: '../config/environmentTest.json'
            },
            srcFolders: ['../mobile', '../backend', "./assets/js", "./config/", "./assets/json", "../backend/config/"]
        },
        zhWords: {
            options: {
                serverUrl: "https://rc-translate.teknisa.com/backend/index.php",
                organization: "20",
                project: "9950"
            }
          }
    });

    grunt.loadNpmTasks('grunt-contrib-jshint');
    grunt.loadNpmTasks('grunt-contrib-jasmine');
    grunt.loadNpmTasks('grunt-contrib-uglify');
    grunt.loadNpmTasks('grunt-contrib-concat');
    grunt.loadNpmTasks('grunt-zh-environment-builder');
    grunt.loadNpmTasks('grunt-zhuacl-structure');
    grunt.loadNpmTasks('grunt-zh-id-generator');
    grunt.loadNpmTasks('grunt-zh-words');

    grunt.registerTask('default', ['jshint', 'concat', 'uglify', 'zhIdGenerator', 'buildAclStruct', 'buildEnvironment:development']);
    grunt.registerTask('test', ['jshint', 'concat', 'uglify', 'buildAclStruct', 'buildEnvironment:test']);
    grunt.registerTask('deploy', ['concat', 'uglify', 'buildAclStruct', 'buildEnvironment:deploy']);
};